@@ -1,7 +1,7 @@
 comboGroups <- function(v, numGroups, retType = "matrix", lower = NULL,
                         upper = NULL, Parallel = FALSE, nThreads = NULL) {
 
-    return(.Call(Algos_ComboGroupsCpp, v, numGroups, retType, lower,
+    return(.Call(`_RcppAlgos_ComboGroupsCpp`, v, numGroups, retType, lower,
                  upper, Parallel, nThreads, pkgEnv$nThreads, FALSE, NULL,
                  NULL, NULL, sample, FALSE, NULL))
 }
@@ -15,11 +15,11 @@ comboGroupsSample <- function(v, numGroups, retType = "matrix",
         set.seed(seed)
     }
 
-    return(.Call(Algos_ComboGroupsCpp, v, numGroups, retType, NULL, NULL,
+    return(.Call(`_RcppAlgos_ComboGroupsCpp`, v, numGroups, retType, NULL, NULL,
                  Parallel, nThreads, pkgEnv$nThreads, TRUE, sampleVec,
                  seed, n, sample, namedSample, new.env()))
 }
 
 comboGroupsCount <- function(v, numGroups) {
-    return(.Call(Algos_ComboGroupsCountCpp, v, numGroups))
+    return(.Call(`_RcppAlgos_ComboGroupsCountCpp`, v, numGroups))
 }
\ No newline at end of file
