@@ -4,9 +4,9 @@
 #include <gmp.h>
 #include <deque>
 
-void nChooseKGmp(mpz_t result, int n, int k);
-void NumCombsWithRepGmp(mpz_t result, int n, int r);
-void MultisetCombRowNumGmp(mpz_t result, int n, int r,
+void nChooseKGmp(mpz_t result, int n, int m);
+void NumCombsWithRepGmp(mpz_t result, int n, int m);
+void MultisetCombRowNumGmp(mpz_t result, int n, int m,
                            const std::deque<int> &Reps);
 
 #endif
