@@ -1,9 +1,7 @@
 #ifndef FUN_ASSIGN_H
 #define FUN_ASSIGN_H
 
-#define R_NO_REMAP
-#include <Rinternals.h>
-#include <R.h>
+#include "cpp11/R.hpp"
 
 void FunAssign(SEXP res, SEXP vectorPass, SEXP sexpFun,
                SEXP rho, int commonType, int commonLen,
