@@ -1,9 +1,8 @@
 #ifndef IMPORT_EXPORT_MPZ_H
 #define IMPORT_EXPORT_MPZ_H
 
-#define R_NO_REMAP
-#include <Rinternals.h>
-#include <R.h>
+#include "cpp11/R.hpp"
+#include "cpp11/protect.hpp"
 
 #include <string>
 #include <gmp.h>
