@@ -10,7 +10,7 @@ double CountPartsDistinctMultiZero(int n, int m, int cap, int strtLen);
 double CountPartsDistinctCapMZ(int n, int m, int cap, int strtLen);
 
 double CountPartsPermDistinct(const std::vector<int> &z,
-                             int tar, int width, bool includeZero);
+                             int tar, int width, bool includeZero = false);
 double CountPartsPermDistinctCap(const std::vector<int> &z, int cap,
                                 int tar, int width, bool includeZero);
 
