@@ -1,9 +1,7 @@
 #ifndef PARTITION_DESIGN_H
 #define PARTITION_DESIGN_H
 
-#define R_NO_REMAP
-#include <Rinternals.h>
-#include <R.h>
+#include "cpp11/R.hpp"
 
 #include "Constraints/ConstraintsTypes.h"
 #include "Partitions/PartitionsTypes.h"
