@@ -1,3 +1,10 @@
+#include "cpp11/logicals.hpp"
+#include "cpp11/integers.hpp"
+#include "cpp11/doubles.hpp"
+#include "cpp11/strings.hpp"
+#include "cpp11/matrix.hpp"
+#include "cpp11/list.hpp"
+
 #include "NumbersUtils/Eratosthenes.h"
 #include "ComboCartesian.h"
 #include "CleanConvert.h"
@@ -21,15 +28,14 @@ void convertToString(std::vector<std::string> &tempVec,
     switch(TYPEOF(ListElement)) {
         case INTSXP: {
             if (bFac) {
-                SEXP facVec = PROTECT(Rf_coerceVector(ListElement, INTSXP));
-                SEXP myLevels = PROTECT(Rf_getAttrib(facVec, R_LevelsSymbol));
+                cpp11::integers facVec(ListElement);
+                cpp11::strings myLevels(facVec.attr("levels"));
                 std::vector<std::string> strVec;
 
                 for (int i = 0, faclen = Rf_length(myLevels); i < faclen; ++i) {
-                    strVec.push_back(CHAR(STRING_ELT(myLevels, i)));
+                    strVec.push_back(myLevels[i]);
                 }
 
-                UNPROTECT(2);
                 typePass = tFac;
 
                 for (auto v: strVec) {
@@ -47,8 +53,7 @@ void convertToString(std::vector<std::string> &tempVec,
                     tempVec.push_back(v);
                 }
             } else {
-                int* intPtr = INTEGER(ListElement);
-                std::vector<int> intVec(intPtr, intPtr + len);
+                cpp11::integers intVec(ListElement);
                 typePass = tInt;
 
                 for (auto v: intVec) {
@@ -58,19 +63,17 @@ void convertToString(std::vector<std::string> &tempVec,
 
             break;
         } case LGLSXP: {
-            int* intPtr = INTEGER(ListElement);
-            std::vector<int> intVec(intPtr, intPtr + len);
+            cpp11::logicals boolVec(ListElement);
             typePass = tLog;
 
-            for (auto v: intVec) {
-                std::string r = std::to_string(v);
+            for (auto v: boolVec) {
+                std::string r = std::to_string(v ? 1 : 0);
                 tempVec.push_back(r + "log");
             }
 
             break;
         } case REALSXP: {
-            double* dblPtr = REAL(ListElement);
-            std::vector<double> dblVec(dblPtr, dblPtr + len);
+            cpp11::doubles dblVec(ListElement);
             typePass = tDbl;
 
             for (auto v: dblVec) {
@@ -114,21 +117,12 @@ void convertToString(std::vector<std::string> &tempVec,
     }
 }
 
-void AddNames(SEXP res, SEXP RList) {
-    if (!Rf_isNull(Rf_getAttrib(RList, R_NamesSymbol))) {
-        SEXP myNames = PROTECT(Rf_getAttrib(RList, R_NamesSymbol));
-        SEXP dimNames = PROTECT(Rf_allocVector(VECSXP, 2));
-        SET_VECTOR_ELT(dimNames, 1, myNames);
-        Rf_setAttrib(res, R_DimNamesSymbol, dimNames);
-        UNPROTECT(2);
-    }
-}
-
-void GetCharOutput(SEXP res, SEXP RList,
+void GetCharOutput(cpp11::writable::strings_matrix<> &res,
                    const std::vector<int> &cartCombs,
                    const std::vector<int> &lastCol,
                    const std::vector<int> &lenGrps,
-                   SEXP charVec, int nCols, int nRows) {
+                   const cpp11::strings &charVec,
+                   int nCols, int nRows) {
 
     for (int i = 0, n1 = nCols - 1, row = 0, m_idx = 0,
          baseSize = lenGrps.size(); i < baseSize; ++i, m_idx = row) {
@@ -136,6 +130,8 @@ void GetCharOutput(SEXP res, SEXP RList,
         for (int j = 0, c_idx = i * n1, grpSize = lenGrps[i];
              j < n1; ++j, ++c_idx, m_idx += nRows) {
 
+            // Benchmarks show that this set up is about 30%
+            // faster than using cpp11::sexp
             SEXP comb = PROTECT(STRING_ELT(charVec, cartCombs[c_idx]));
 
             for (int k = 0; k < grpSize; ++k) {
@@ -150,36 +146,60 @@ void GetCharOutput(SEXP res, SEXP RList,
                            STRING_ELT(charVec, lastCol[row]));
         }
     }
-
-    AddNames(res, RList);
 }
 
-template <typename T>
-void GetPureOutput(T* mat, SEXP res, SEXP RList,
+template <typename matType, typename T>
+void GetPureOutput(matType &mat,
                    const std::vector<int> &cartCombs,
                    const std::vector<int> &lastCol,
                    const std::vector<int> &lenGrps,
-                   const T* standardVec, int nCols, int nRows) {
+                   const std::vector<T> &standardVec,
+                   int nCols, int nRows) {
 
     for (int i = 0, n1 = nCols - 1, row = 0, m_idx = 0,
-         baseSize = lenGrps.size(); i < baseSize; ++i, m_idx = row) {
+         baseSize = lenGrps.size(); i < baseSize; ++i) {
 
         for (int j = 0, c_idx = i * n1, grpSize = lenGrps[i];
-             j < n1; ++j, ++c_idx, m_idx += nRows) {
+             j < n1; ++j, ++c_idx) {
 
             auto&& comb = standardVec[cartCombs[c_idx]];
 
             for (int k = 0; k < grpSize; ++k) {
-                mat[m_idx + k] = comb;
+                mat(m_idx + k, j) = comb;
             }
         }
 
         for (int k = 0; k < lenGrps[i]; ++k, ++row) {
-            mat[m_idx + k] = standardVec[lastCol[row]];
+            mat(m_idx + k, n1) = standardVec[lastCol[row]];
+        }
+
+        m_idx += lenGrps[i];
+    }
+}
+
+template <typename cpp11Type, typename T>
+void PoulateColumn(const std::vector<int> &cartCombs,
+                   const std::vector<int> &lastCol,
+                   const std::vector<int> &lenGrps,
+                   const std::vector<T> &poolVec,
+                   cpp11Type &res, int nCols, int nRows, int i) {
+
+    if (i < (nCols - 1)) {
+        for (int j = 0, n1 = nCols - 1, row = i, idx = 0,
+             baseSize = lenGrps.size(); idx < baseSize;
+             ++idx, row += n1) {
+
+            auto&& comb = poolVec[cartCombs[row]];
+
+            for (int k = 0; k < lenGrps[idx]; ++k, ++j) {
+                res[j] = comb;
+            }
+        }
+    } else {
+        for (int j = 0; j < nRows; ++j) {
+            res[j] = poolVec[lastCol[j]];
         }
     }
-
-    AddNames(res, RList);
 }
 
 SEXP GlueComboCart(const std::vector<int> &cartCombs,
@@ -187,122 +207,52 @@ SEXP GlueComboCart(const std::vector<int> &cartCombs,
                    const std::vector<int> &lenGrps,
                    const std::vector<std::vector<int>> &facList,
                    const std::vector<int> &typeCheck,
-                   const std::vector<int> &IsFactor, SEXP RList,
-                   int* intVec, int* boolVec, double* dblVec,
-                   SEXP charVec, int nRows, int nCols, bool IsDF) {
+                   const std::vector<int> &IsFactor,
+                   const cpp11::list &RList,
+                   const std::vector<int> &intVec,
+                   const std::vector<double> &dblVec,
+                   const std::vector<int> &boolVec,
+                   const cpp11::strings &charVec,
+                   int nRows, int nCols, bool IsDF) {
 
     if (IsDF) {
-        SEXP DataFrame = PROTECT(Rf_allocVector(VECSXP, nCols));
-        int numProtects = 1;
+        cpp11::writable::list DataFrame(nCols);
 
         for (int i = 0, facInd = 0; i < nCols; ++i) {
-            switch (TYPEOF(VECTOR_ELT(RList, i))) {
+            switch (TYPEOF(RList[i])) {
                 case INTSXP: {
-                    SEXP sexpVec = PROTECT(Rf_allocVector(INTSXP, nRows));
-                    int* intSexpVec = INTEGER(sexpVec);
-                    ++numProtects;
+                    cpp11::writable::integers intSexpVec(nRows);
 
                     if (IsFactor[i]) {
-                        const int size = facList[facInd].size();
-                        SEXP facVec = PROTECT(Rf_allocVector(INTSXP, size));
-                        int* intFacVec = INTEGER(facVec);
-                        ++numProtects;
+                        const std::vector<int> facVec(
+                            facList[facInd].begin(), facList[facInd].end()
+                        );
 
-                        for (int j = 0; j < size; ++j) {
-                            intFacVec[j] = facList[facInd][j];
-                        }
-
-                        if (i < (nCols - 1)) {
-                            for (int j = 0, n1 = nCols - 1, row = i, idx = 0,
-                                 baseSize = lenGrps.size(); idx < baseSize;
-                                 ++idx, row += n1) {
-
-                                auto&& comb = intFacVec[cartCombs[row]];
-
-                                for (int k = 0; k < lenGrps[idx]; ++k, ++j) {
-                                    intSexpVec[j] = comb;
-                                }
-                            }
-                        } else {
-                            for (int j = 0; j < nRows; ++j) {
-                                intSexpVec[j] = intFacVec[lastCol[j]];
-                            }
-                        }
-
-                        SetFactorClass(sexpVec, VECTOR_ELT(RList, i));
+                        PoulateColumn(cartCombs, lastCol, lenGrps,
+                                      facVec, intSexpVec, nCols, nRows, i);
+                        SetFactorClass(intSexpVec, RList[i]);
                         ++facInd;
                     } else {
-                        if (i < (nCols - 1)) {
-                            for (int j = 0, n1 = nCols - 1, row = i, idx = 0,
-                                 baseSize = lenGrps.size(); idx < baseSize;
-                                 ++idx, row += n1) {
-
-                                auto&& comb = intVec[cartCombs[row]];
-
-                                for (int k = 0; k < lenGrps[idx]; ++k, ++j) {
-                                    intSexpVec[j] = comb;
-                                }
-                            }
-                        } else {
-                            for (int j = 0; j < nRows; ++j) {
-                                intSexpVec[j] = intVec[lastCol[j]];
-                            }
-                        }
+                        PoulateColumn(cartCombs, lastCol, lenGrps,
+                                      intVec, intSexpVec, nCols, nRows, i);
                     }
 
-                    SET_VECTOR_ELT(DataFrame, i, sexpVec);
+                    DataFrame[i] = intSexpVec;
                     break;
                 } case LGLSXP: {
-                    SEXP sexpVec = PROTECT(Rf_allocVector(LGLSXP, nRows));
-                    int* boolSexpVec = INTEGER(sexpVec);
-                    ++numProtects;
-
-                    if (i < (nCols - 1)) {
-                        for (int j = 0, n1 = nCols - 1, row = i, idx = 0,
-                             baseSize = lenGrps.size(); idx < baseSize;
-                             ++idx, row += n1) {
-
-                            auto&& comb = boolVec[cartCombs[row]];
-
-                            for (int k = 0; k < lenGrps[idx]; ++k, ++j) {
-                                boolSexpVec[j] = comb;
-                            }
-                        }
-                    } else {
-                        for (int j = 0; j < nRows; ++j) {
-                            boolSexpVec[j] = boolVec[lastCol[j]];
-                        }
-                    }
-
-                    SET_VECTOR_ELT(DataFrame, i, sexpVec);
+                    cpp11::writable::logicals boolSexpVec(nRows);
+                    PoulateColumn(cartCombs, lastCol, lenGrps,
+                                  boolVec, boolSexpVec, nCols, nRows, i);
+                    DataFrame[i] = boolSexpVec;
                     break;
                 } case REALSXP: {
-                    SEXP sexpVec = PROTECT(Rf_allocVector(REALSXP, nRows));
-                    double* dblSexpVec = REAL(sexpVec);
-                    ++numProtects;
-
-                    if (i < (nCols - 1)) {
-                        for (int j = 0, n1 = nCols - 1, row = i, idx = 0,
-                             baseSize = lenGrps.size(); idx < baseSize;
-                             ++idx, row += n1) {
-
-                            auto&& comb = dblVec[cartCombs[row]];
-
-                            for (int k = 0; k < lenGrps[idx]; ++k, ++j) {
-                                dblSexpVec[j] = comb;
-                            }
-                        }
-                    } else {
-                        for (int j = 0; j < nRows; ++j) {
-                            dblSexpVec[j] = dblVec[lastCol[j]];
-                        }
-                    }
-
-                    SET_VECTOR_ELT(DataFrame, i, sexpVec);
+                    cpp11::writable::doubles dblSexpVec(nRows);
+                    PoulateColumn(cartCombs, lastCol, lenGrps,
+                                  dblVec, dblSexpVec, nCols, nRows, i);
+                    DataFrame[i] = dblSexpVec;
                     break;
                 } case STRSXP: {
-                    SEXP sexpVec = PROTECT(Rf_allocVector(STRSXP, nRows));
-                    ++numProtects;
+                    cpp11::writable::strings sexpVec(nRows);
 
                     if (i < (nCols - 1)) {
                         for (int j = 0, n1 = nCols - 1, row = i, idx = 0,
@@ -324,59 +274,42 @@ SEXP GlueComboCart(const std::vector<int> &cartCombs,
                         }
                     }
 
-                    SET_VECTOR_ELT(DataFrame, i, sexpVec);
+                    DataFrame[i] = sexpVec;
                     break;
                 }
             }
         }
 
-        Rf_setAttrib(DataFrame, R_ClassSymbol, Rf_mkString("data.frame"));
-        Rf_setAttrib(DataFrame, R_NamesSymbol,
-                     Rf_getAttrib(RList, R_NamesSymbol));
-        SEXP rownames = PROTECT(Rf_allocVector(INTSXP, nRows));
-        int* intRows  = INTEGER(rownames);
-
-        for (int i = 1; i <= nRows; ++i) {
-            intRows[i - 1] = i;
-        }
-
-        Rf_setAttrib(DataFrame, R_RowNamesSymbol, rownames);
-        UNPROTECT(numProtects + 1);
+        DataFrame.attr("row.names") = {NA_INTEGER, -nRows};
+        DataFrame.names() = RList.names();
+        DataFrame.attr("class") = "data.frame";
         return DataFrame;
     } else {
         if (typeCheck[tInt]) {
-            SEXP res = PROTECT(Rf_allocMatrix(INTSXP, nRows, nCols));
-            int* intMat = INTEGER(res);
-            GetPureOutput(intMat, res, RList, cartCombs, lastCol,
+            cpp11::writable::integers_matrix<> intMat(nRows, nCols);
+            GetPureOutput(intMat, cartCombs, lastCol,
                           lenGrps, intVec, nCols, nRows);
-            UNPROTECT(1);
-            return res;
+            return intMat;
         } else if (typeCheck[tFac]) {
-            SEXP res = PROTECT(Rf_allocMatrix(INTSXP, nRows, nCols));
-            int* intMat = INTEGER(res);
-            GetPureOutput(intMat, res, RList, cartCombs, lastCol,
+            cpp11::writable::integers_matrix<> intMat(nRows, nCols);
+            GetPureOutput(intMat, cartCombs, lastCol,
                           lenGrps, intVec, nCols, nRows);
-            SetFactorClass(res, VECTOR_ELT(RList, 0));
-            UNPROTECT(1);
-            return res;
+            SetFactorClass(intMat, RList[0]);
+            return intMat;
         } else if (typeCheck[tLog]) {
-            SEXP res = PROTECT(Rf_allocMatrix(LGLSXP, nRows, nCols));
-            int* intMat = INTEGER(res);
-            GetPureOutput(intMat, res, RList, cartCombs, lastCol,
+            cpp11::writable::logicals_matrix<> boolMat(nRows, nCols);
+            GetPureOutput(boolMat, cartCombs, lastCol,
                           lenGrps, boolVec, nCols, nRows);
-            UNPROTECT(1);
-            return res;
+            return boolMat;
         } else if (typeCheck[tDbl]) {
-            SEXP res = Rf_allocMatrix(REALSXP, nRows, nCols);
-            double* dblMat = REAL(res);
-            GetPureOutput(dblMat, res, RList, cartCombs, lastCol,
+            cpp11::writable::doubles_matrix<> dblMat(nRows, nCols);
+            GetPureOutput(dblMat, cartCombs, lastCol,
                           lenGrps, dblVec, nCols, nRows);
-            return res;
+            return dblMat;
         } else {
-            SEXP res = PROTECT(Rf_allocMatrix(STRSXP, nRows, nCols));
-            GetCharOutput(res, RList, cartCombs, lastCol,
+            cpp11::writable::strings_matrix<> res(nRows, nCols);
+            GetCharOutput(res, cartCombs, lastCol,
                           lenGrps, charVec, nCols, nRows);
-            UNPROTECT(1);
             return res;
         }
     }
@@ -402,20 +335,20 @@ void getAtLeastNPrimes(std::vector<int> &primes,
 }
 
 [[cpp11::register]]
-SEXP ComboGridCpp(SEXP RList, SEXP RIsRep) {
+SEXP ComboGridCpp(cpp11::list RList, bool IsRep) {
 
     int sumLength = 0;
     const int nCols = Rf_length(RList);
     std::vector<int> IsFactor(nCols);
 
     for (int i = 0; i < nCols; ++i) {
-        if (Rf_isFactor(VECTOR_ELT(RList, i))) {
+        if (Rf_isFactor(RList[i])) {
             IsFactor[i] = 1;
         } else {
             IsFactor[i] = 0;
         }
 
-        sumLength += Rf_length(VECTOR_ELT(RList, i));
+        sumLength += Rf_length(RList[i]);
     }
 
     std::vector<int> primes;
@@ -423,7 +356,6 @@ SEXP ComboGridCpp(SEXP RList, SEXP RIsRep) {
 
     int numFactorVec = std::accumulate(IsFactor.cbegin(),
                                        IsFactor.cend(), 0);
-    const int IsRep = CleanConvert::convertFlag(RIsRep, "IsRep");
 
     // All duplicates have been removed from RList via
     // lapply(RList, function(x) sort(unique(x)))
@@ -431,14 +363,10 @@ SEXP ComboGridCpp(SEXP RList, SEXP RIsRep) {
     std::unordered_map<std::string, int> mapIndex;
     std::vector<int> typeCheck(5, 0);
 
-    SEXP charVec     = PROTECT(Rf_allocVector(STRSXP, sumLength));
-    SEXP dblSexpVec  = PROTECT(Rf_allocVector(REALSXP, sumLength));
-    SEXP intSexpVec  = PROTECT(Rf_allocVector(INTSXP, sumLength));
-    SEXP boolSexpVec = PROTECT(Rf_allocVector(LGLSXP, sumLength));
-
-    double* dblVec = REAL(dblSexpVec);
-    int* intVec    = INTEGER(intSexpVec);
-    int* boolVec   = INTEGER(boolSexpVec);
+    cpp11::writable::strings charVec(sumLength);
+    std::vector<double> dblVec(sumLength);
+    std::vector<int> intVec(sumLength);
+    std::vector<int> boolVec(sumLength);
 
     std::vector<std::vector<int>> facList(numFactorVec,
                                           std::vector<int>(sumLength, 0));
@@ -446,7 +374,7 @@ SEXP ComboGridCpp(SEXP RList, SEXP RIsRep) {
     for (int i = 0, total = 0, myIndex = 0, facInd = 0; i < nCols; ++i) {
         rcppType myType;
         std::vector<std::string> tempVec;
-        convertToString(tempVec, VECTOR_ELT(RList, i), myType, IsFactor[i]);
+        convertToString(tempVec, RList[i], myType, IsFactor[i]);
         std::size_t j = 0;
 
         for (const auto &v: tempVec) {
@@ -468,23 +396,23 @@ SEXP ComboGridCpp(SEXP RList, SEXP RIsRep) {
 
             switch(myType) {
                 case tInt: {
-                    intVec[myIndex] = INTEGER(VECTOR_ELT(RList, i))[j];
+                    intVec[myIndex] = INTEGER(RList[i])[j];
                     typeCheck[tInt] = 1;
                     break;
                 } case tFac: {
-                    facList[facInd][myIndex] = INTEGER(VECTOR_ELT(RList, i))[j];
+                    facList[facInd][myIndex] = INTEGER(RList[i])[j];
                     typeCheck[tFac] = 1;
                     break;
                 } case tDbl: {
-                    dblVec[myIndex] = REAL(VECTOR_ELT(RList, i))[j];
+                    dblVec[myIndex] = REAL(RList[i])[j];
                     typeCheck[tDbl] = 1;
                     break;
                 } case tStr: {
-                    SET_STRING_ELT(charVec, myIndex, STRING_ELT(VECTOR_ELT(RList, i), j));
+                    SET_STRING_ELT(charVec, myIndex, STRING_ELT(RList[i], j));
                     typeCheck[tStr] = 1;
                     break;
                 } case tLog: {
-                    boolVec[myIndex] = INTEGER(VECTOR_ELT(RList, i))[j];
+                    boolVec[myIndex] = LOGICAL(RList[i])[j];
                     typeCheck[tLog] = 1;
                     break;
                 }
@@ -504,19 +432,14 @@ SEXP ComboGridCpp(SEXP RList, SEXP RIsRep) {
 
         for (int i = 0; i < nCols; ++i) {
             if (IsFactor[i]) {
-                SEXP facVec = PROTECT(Rf_getAttrib(VECTOR_ELT(RList, i),
-                                                   R_LevelsSymbol));
+                cpp11::strings facVec(Rf_getAttrib(RList[i], R_LevelsSymbol));
                 std::vector<std::string> strVec;
 
-                int len_comp = Rf_length(facVec);
-
-                for (int i = 0; i < len_comp; ++i) {
-                    const std::string temp(CHAR(STRING_ELT(facVec, i)));
+                for (auto f: facVec) {
+                    const std::string temp(CHAR(f));
                     strVec.push_back(temp);
                 }
 
-                UNPROTECT(1);
-
                 if (testLevels.size()) {
                     if (strVec != testLevels) {
                         ++mySum;
@@ -536,9 +459,8 @@ SEXP ComboGridCpp(SEXP RList, SEXP RIsRep) {
     comboGrid(cartCombs, lastCol, lenGrps, myVec, primes, IsRep);
 
     const int nRows = lastCol.size();
-    SEXP res =  PROTECT(GlueComboCart(cartCombs, lastCol, lenGrps, facList,
-                                      typeCheck, IsFactor, RList, intVec,
-                                      boolVec, dblVec, charVec, nRows, nCols, IsDF));
-    UNPROTECT(5);
+    cpp11::sexp res = GlueComboCart(cartCombs, lastCol, lenGrps, facList,
+                                    typeCheck, IsFactor, RList, intVec,
+                                    dblVec, boolVec, charVec, nRows, nCols, IsDF);
     return res;
 }
